from __future__ import annotations

from typing import Any, Iterable

from psnawp_api.models.group import Group
from psnawp_api.models.user import User
from psnawp_api.utils.endpoints import BASE_PATH, API_PATH
from psnawp_api.utils.request_builder import RequestBuilder


class Client:
    """The Client class provides the information and methods for the currently authenticated user."""

    def __init__(self, request_builder: RequestBuilder):
        """Initialize a Client instance.

        .. note::

            This class is intended to be interfaced with through PSNAWP.

        :param request_builder: The instance of RequestBuilder. Used to make
            HTTPRequests.
        :type request_builder: RequestBuilder

        """
        self._request_builder = request_builder

    @property
    def online_id(self) -> str:
        """Gets the online ID of the client logged in the api.

        :returns: onlineID
        :rtype: str

        .. code-block:: Python

            client = psnawp.me()
            print(client.online_id)

        """
        response = self._request_builder.get(
            url=f"{BASE_PATH['profile_uri']}{API_PATH['profiles'].format(account_id=self.account_id)}"
        ).json()
        online_id: str = response["onlineId"]
        return online_id

    @property
    def account_id(self) -> str:
        """Gets the account ID of the client logged in the api.

        :returns: accountID
        :rtype: str

        .. code-block:: Python

            client = psnawp.me()
            print(client.account_id)

        """
        response = self._request_builder.get(
            url=f"{BASE_PATH['account_uri']}{API_PATH['my_account']}"
        ).json()
        account_id: str = response["accountId"]
        return account_id

    def get_profile_legacy(self) -> dict[str, Any]:
        """Gets the profile info from legacy api endpoint. Useful for legacy console (PS3, PS4) presence.

        :returns: A dict containing info similar to what is shown below:
        :rtype: dict[str, Any]

            .. code-block:: json

                {
                    "profile": {
                        "onlineId": "VaultTec_Trading",
                        "accountId": "8520698476712646544",
                        "npId": "VmF1bHRUZWNfVHJhZGluZ0BiNi51cw==",
                        "avatarUrls": [
                            {
                                "size": "l",
                                "avatarUrl": "[Redacted]"
                            }
                        ],
                        "plus": 0,
                        "aboutMe": "r/Fallout76Marketplace Moderator",
                        "languagesUsed": [
                            "en"
                        ],
                        "trophySummary": {
                            "level": 1,
                            "progress": 0,
                            "earnedTrophies": {
                                "platinum": 0,
                                "gold": 0,
                                "silver": 0,
                                "bronze": 0
                            }
                        },
                        "isOfficiallyVerified": false,
                        "personalDetail": {
                            "firstName": "VaultTec",
                            "lastName": "TradingCo",
                            "profilePictureUrls": [
                                {
                                    "size": "xl",
                                    "profilePictureUrl": "[Redacted]"
                                }
                            ]
                        },
                        "personalDetailSharing": "no",
                        "personalDetailSharingRequestMessageFlag": false,
                        "primaryOnlineStatus": "offline",
                        "presences": [
                            {
                                "onlineStatus": "offline",
                                "hasBroadcastData": false,
                                "lastOnlineDate": "2021-02-05T20:14:45Z"
                            }
                        ],
                        "friendRelation": "no",
                        "requestMessageFlag": false,
                        "blocking": false,
                        "following": false,
                        "consoleAvailability": {
                            "availabilityStatus": "offline"
                        }
                    }
                }


        .. code-block:: Python

            client = psnawp.me()
            print(client.get_profile_legacy())

        """
        params = {
            "fields": "npId,onlineId,accountId,avatarUrls,plus,aboutMe,languagesUsed,trophySummary(@default,level,progress,earnedTrophies),"
            "isOfficiallyVerified,personalDetail(@default,profilePictureUrls),personalDetailSharing,personalDetailSharingRequestMessageFlag,"
            "primaryOnlineStatus,presences(@default,@titleInfo,platform,lastOnlineDate,hasBroadcastData),requestMessageFlag,blocking,friendRelation,"
            "following,consoleAvailability"
        }

        response: dict[str, Any] = self._request_builder.get(
            url=f"{BASE_PATH['legacy_profile_uri']}{API_PATH['legacy_profile'].format(online_id=self.online_id)}",
            params=params,
        ).json()

        return response

    def get_account_devices(self) -> list[dict[str, Any]]:
        """Gets the list of devices the client is logged into.

        :returns: A dict containing info similar to what is shown below:
        :rtype: list[dict[str, Any]]

            .. code-block:: json

                [
                    {
                        "deviceId": "[Redacted]",
                        "deviceType": "PS4",
                        "activationType": "PSN_GAME_V3",
                        "activationDate": "2021-02-05T20:11:27.815Z",
                        "accountDeviceVector": "[Redacted]"
                    }
                ]


        .. code-block:: Python

            client = psnawp.me()
            print(client.get_account_devices())

        """
        params = {
            "includeFields": "device,systemData",
            "platform": "PS5,PS4,PS3,PSVita",
        }
        response = self._request_builder.get(
            url=f"{BASE_PATH['account_uri']}{API_PATH['my_account']}", params=params
        ).json()

        # Just so mypy doesn't complain
        account_devices: list[dict[str, Any]] = response.get("accountDevices", [])
        return account_devices

    def friends_list(self, limit: int = 1000) -> Iterable[User]:
        """Gets the friends list and return their account ids.

        :param limit: The number of items from input max is 1000.
        :type limit: int

        :returns: All friends in your friends list.
        :rtype: Iterable[User]

        .. code-block:: Python

            client = psnawp.me()
            friends_list = client.friends_list()

            for friend in friends_list:
                ...

        """
        limit = min(1000, limit)

        params = {"limit": limit}
        response = self._request_builder.get(
            url=f"{BASE_PATH['profile_uri']}{API_PATH['friends_list']}", params=params
        ).json()
        return (
            User(
                request_builder=self._request_builder,
                account_id=account_id,
                online_id=None,
            )
            for account_id in response["friends"]
        )

    def available_to_play(self) -> Iterable[User]:
        """Gets the list of users on your "Notify when available" subscription list.

        :returns: Iterable of user objects.
        :rtype: Iterable[User]

        .. code-block:: Python

            client = psnawp.me()
            available_to_play = client.available_to_play()

            for user in available_to_play:
                ...

        """
        response = self._request_builder.get(
            url=f"{BASE_PATH['profile_uri']}{API_PATH['available_to_play']}"
        ).json()
        return (
            User(
                request_builder=self._request_builder,
                account_id=account_id_dict["accountId"],
                online_id=None,
            )
            for account_id_dict in response["settings"]
        )

    def blocked_list(self) -> Iterable[User]:
        """Gets the blocked list and return their account ids.

        :returns: Al blocked users on your block list.
        :rtype: Iterable[User]

        .. code-block:: Python

            client = psnawp.me()
            blocked_list = client.blocked_list()

            for blocked_users in blocked_list:
                ...

        """
        response = self._request_builder.get(
            url=f"{BASE_PATH['profile_uri']}{API_PATH['blocked_users']}"
        ).json()
        return (
            User(
                request_builder=self._request_builder,
                account_id=account_id,
                online_id=None,
            )
            for account_id in response["blockList"]
        )

    def get_groups(self, limit: int = 200, offset: int = 0) -> Iterable[Group]:
        """Gets all the groups you have participated in.

        :param limit: The number of groups to receive.
        :type limit: int
        :param offset: Lets you exclude first N items groups. Offset = 10 lets you skip
            the first 10 groups.
        :type offset: int

        :returns: Iterable of Group Objects.
        :rtype: Iterable[Group]

        """
        param = {"includeFields": "members", "limit": limit, "offset": offset}

        response = self._request_builder.get(
            url=f"{BASE_PATH['gaming_lounge']}{API_PATH['my_groups']}", params=param
        ).json()

        return (
            Group(
                request_builder=self._request_builder,
                group_id=group_info["groupId"],
                users=None,
            )
            for group_info in response["groups"]
        )

    def __repr__(self):
        return f"<User online_id:{self.online_id} account_id:{self.account_id}>"

    def __str__(self):
        return f"Online ID: {self.online_id} Account ID: {self.account_id}"



from __future__ import annotations

import logging
import re
from typing import overload, Optional, Iterable

from psnawp_api.core import authenticator
from psnawp_api.core.psnawp_exceptions import PSNAWPIllegalArgumentError
from psnawp_api.models.client import Client
from psnawp_api.models.group import Group
from psnawp_api.models.search import Search
from psnawp_api.models.user import User
from psnawp_api.utils import request_builder

logging_level = logging.INFO


class PSNAWP:
    """PlayStation Network API Wrapper Python (PSNAWP) Retrieve User Information, Trophies, Game and Store data from the PlayStation Network.

    Instances of this class are the gateway to interacting with PSN API through PSNAWP.

    .. code-block:: Python

        from psnawp_api import PSNAWP
        psnawp = PSNAWP('<64 character npsso code>')

    """

    def __init__(self, npsso_cookie):
        """Constructor Method. Takes the npsso_cookie and creates instance of ``request_builder.RequestBuilder`` which is used later in code for HTTPS requests.

        :param npsso_cookie: npsso cookie obtained from PSN website.
        :type npsso_cookie: str

        :raises: ``PSNAWPAuthenticationError`` If npsso code is expired or is incorrect.

        """
        self._request_builder = request_builder.RequestBuilder(
            authenticator.Authenticator(npsso_cookie)
        )

    def me(self) -> Client:
        """Creates a new client object (your account).

        :returns: Client Object
        :rtype: Client

        .. code-block:: Python

            from psnawp_api import PSNAWP
            psnawp = PSNAWP('<64 character npsso code>')
            client = psnawp.me()

        """
        return Client(self._request_builder)

    @overload
    def user(self, *, online_id: str) -> User:
        ...

    @overload
    def user(self, *, account_id: str) -> User:
        ...

    def user(self, **kwargs):
        """Creates a new user object using Online ID (GamerTag) or Account ID (PSN ID).

        .. note::

            You may only provide Online ID or Account ID. But not both at once.

        :param kwargs: online_id (str): Online ID (GamerTag) of the user. account_id
            (str): Account ID of the user.
        :type kwargs: dict

        :returns: User Object
        :rtype: User

        :raises: `PSNAWPIllegalArgumentError` If None or Both kwargs are passed.

        :raises: ``PSNAWPNotFound`` If the user is not valid/found.

        .. code-block:: Python

            user1 = psnawp.user(online_id="VaultTec_Trading")
            user2 = psnawp.user(account_id='1802043923080044300')

        """
        online_id: Optional[str] = kwargs.get("online_id")
        account_id: Optional[str] = kwargs.get("account_id")

        if (online_id and account_id) or not (online_id or account_id):
            raise PSNAWPIllegalArgumentError(
                "You provide at least online ID or account ID, and not both."
            )

        if account_id is not None and not re.match(r"\d{19}", account_id):
            raise PSNAWPIllegalArgumentError(
                "The account id is not correct. Perhaps you meant online_id?"
            )
        return User(self._request_builder, online_id, account_id)

    @overload
    def group(self, *, group_id: str) -> Group:
        ...

    @overload
    def group(self, *, users_list: Iterable[User]) -> Group:
        ...

    def group(self, **kwargs):
        """Creates a group object from a Group ID or from list of users.

        .. warning::

            Passing ``users_list`` will create a new group each time. If you want to
            continue from the same group. Use group id obtained from
            ``client.get_groups()``

        :param kwargs: group_id (str): The Group ID of a group usually retrieved with
            the get_groups() method. users_list(Iterable[User]): A list of users of the
            members in the group.

        :returns: Group Object
        :rtype: Group

        :raises: ``PSNAWPIllegalArgumentError`` If None or Both kwargs are passed.

        :raises: ``PSNAWPForbidden`` If you are Dming a user who has blocked you.

        """

        group_id: Optional[str] = kwargs.get("group_id")
        users: Optional[Iterable[User]] = kwargs.get("users_list")

        if (group_id and users) or not (group_id or users):
            raise PSNAWPIllegalArgumentError(
                "You provide at least Group Id or Users, and not both."
            )
        return Group(self._request_builder, group_id=group_id, users=users)

    def search(self) -> Search:
        """Creates a new search object

        :returns: Search Object

        """
        return Search(self._request_builder)



from __future__ import annotations

from typing import Optional, Any

from psnawp_api.core.psnawp_exceptions import (
    PSNAWPNotFound,
    PSNAWPForbidden,
    PSNAWPBadRequest,
)
from psnawp_api.utils.endpoints import BASE_PATH, API_PATH
from psnawp_api.utils.request_builder import RequestBuilder


class User:
    """This class will contain the information about the PSN ID you passed in when creating object"""

    def __init__(
        self,
        request_builder: RequestBuilder,
        online_id: Optional[str],
        account_id: Optional[str],
    ):
        """Constructor of Class User. Creates user object using online id or account id.

        .. note::

            This class is intended to be interfaced with through PSNAWP.

        :param request_builder: Used to call http requests.
        :type request_builder: RequestBuilder
        :param online_id: Online ID (GamerTag) of the user.
        :type online_id: str
        :param account_id: Account ID of the user.
        :type account_id: str

        :raises: ``PSNAWPIllegalArgumentError`` If both online_id and account_id are not
            provided.

        :raises: ``PSNAWPNotFound`` If the user is not valid/found.

        """
        self._request_builder = request_builder
        self.online_id = online_id
        self.account_id = account_id
        self._prev_online_id = online_id

        if self.online_id is not None:
            profile = self._online_id_to_account_id()
            self.account_id = profile["profile"]["accountId"]
            self.online_id = profile["profile"].get(
                "currentOnlineId", profile["profile"]["onlineId"]
            )
        elif self.account_id is not None:
            profile = self.profile()
            self.online_id = profile["onlineId"]

    @property
    def prev_online_id(self) -> str:
        """Gets the previous online ID of the user.

        Note: Playstation allows you to look up a user using old online id as long as it
        is not taken by another player. This might be same as ``online_id`` depending on
        user.

        .. note::

            If you are initializing User Object using Account ID then the previous
            online id will be same as online id. This is just the limitation of API.

        :returns: onlineID
        :rtype: str

        .. code-block:: Python

            user_example = psnawp.user(online_id='VaultTec_Trading')
            print(user_example.prev_online_id)

        """
        profile = self._online_id_to_account_id(prev_online_id=self._prev_online_id)[
            "profile"
        ]
        prev_online_id: str = profile.get("onlineId")
        return prev_online_id

    def _online_id_to_account_id(
        self, prev_online_id: Optional[str] = None
    ) -> dict[str, Any]:
        """Converts user online ID and returns their account id. This is an internal function and not meant to be called directly.

        :returns: dict: PSN ID and Account ID of the user in search query
        :rtype: dict[str, Any]

        :raises: ``PSNAWPNotFound`` If the user is not valid/found.

        """

        online_id = self.online_id
        if prev_online_id is not None:
            online_id = prev_online_id

        try:
            query = {"fields": "accountId,onlineId,currentOnlineId"}
            response: dict[str, Any] = self._request_builder.get(
                url=f"{BASE_PATH['legacy_profile_uri']}{API_PATH['legacy_profile'].format(online_id=online_id)}",
                params=query,
            ).json()
            return response
        except PSNAWPNotFound as not_found:
            raise PSNAWPNotFound(
                f"Online ID {self.online_id} does not exist."
            ) from not_found

    def profile(self) -> dict[str, Any]:
        """Gets the profile of the user such as about me, avatars, languages etc...

        :returns: A dict containing info similar to what is shown below:
        :rtype: dict[str, Any]

            .. code-block:: json

                {
                    "onlineId": "VaultTec-Co",
                    "aboutMe": "r/Fallout76Marketplace Moderator",
                    "avatars": [
                        {
                            "size": "s",
                            "url": "[Redacted]"
                        },
                        {
                            "size": "xl",
                            "url": "[Redacted]"
                        },
                        {
                            "size": "l",
                            "url": "[Redacted]"
                        },
                        {
                            "size": "m",
                            "url": "[Redacted]"
                        }
                    ],
                    "languages": [
                        "en-US"
                    ],
                    "isPlus": false,
                    "isOfficiallyVerified": false,
                    "isMe": false
                }


        :raises: ``PSNAWPNotFound`` If the user is not valid/found.

        .. code-block:: Python

            user_example = psnawp.user(online_id='VaultTec_Trading')
            print(user_example.profile())

        """

        try:
            response: dict[str, Any] = self._request_builder.get(
                url=f"{BASE_PATH['profile_uri']}{API_PATH['profiles'].format(account_id=self.account_id)}"
            ).json()
            return response
        except PSNAWPBadRequest as bad_request:
            raise PSNAWPNotFound(
                f"Account ID {self.account_id} does not exist."
            ) from bad_request

    def get_presence(self) -> dict[str, Any]:
        """Gets the presences of a user. If the profile is private

        :returns: A dict containing info similar to what is shown below:
        :rtype: dict[str, Any]

            .. code-block:: json

                {
                    "availability": "availableToPlay",
                    "primaryPlatformInfo": {
                        "onlineStatus": "online",
                        "platform": "ps4",
                        "lastOnlineDate": "2022-09-15T22:50:28.012Z"
                    }
                }


        :raises: ``PSNAWPForbidden`` When the user's profile is private, and you don't
            have permission to view their online status.

        .. code-block:: Python

            user_example = psnawp.user(online_id='VaultTec_Trading')
            print(user_example.get_presence())

        """
        try:
            params = {"type": "primary"}
            response: dict[str, Any] = self._request_builder.get(
                url=f"{BASE_PATH['profile_uri']}/{self.account_id}{API_PATH['basic_presences']}",
                params=params,
            ).json()
            return response
        except PSNAWPForbidden as forbidden:
            raise PSNAWPForbidden(
                f"You are not allowed to check the presence of user {self.online_id}"
            ) from forbidden

    def friendship(self) -> dict[str, Any]:
        """Gets the friendship status and stats of the user

        :returns: A dict containing info similar to what is shown below
        :rtype: dict[str, Any]

            .. code-block:: json

                {
                    "friendRelation": "friend",
                    "personalDetailSharing": "none",
                    "friendsCount": 419,
                    "mutualFriendsCount": 0
                }


        .. code-block:: Python

            user_example = psnawp.user(online_id='VaultTec_Trading')
            print(user_example.friendship())

        """
        response: dict[Any, Any] = self._request_builder.get(
            url=f"{BASE_PATH['profile_uri']}{API_PATH['friends_summary'].format(account_id=self.account_id)}"
        ).json()
        return response

    def is_blocked(self) -> bool:
        """Checks if the user is blocked by you

        :returns: True if the user is blocked otherwise False
        :rtype: bool

        .. code-block:: Python

            user_example = psnawp.user(online_id='VaultTec_Trading')
            print(user_example.is_blocked())

        """
        response = self._request_builder.get(
            url=f"{BASE_PATH['profile_uri']}{API_PATH['blocked_users']}"
        ).json()
        return self.account_id in response["blockList"]

    def __repr__(self):
        return f"<User online_id:{self.online_id} account_id:{self.account_id}>"

    def __str__(self):
        return f"Online ID: {self.online_id} Account ID: {self.account_id}"



from __future__ import annotations

import logging
from psnawp_api import psnawp


def create_logger(module_name: str) -> logging.Logger:
    """Creates logger and returns an instance of logging object.

    :param module_name: Logger name that will appear in text.
    :type module_name: str

    :returns: Logging Object.
    :rtype: logging.Logger

    """
    # Setting up the root logger
    logger = logging.getLogger(module_name)
    logger.setLevel(logging.DEBUG)

    log_stream = logging.StreamHandler()
    log_stream.setLevel(psnawp.logging_level)
    formatter = logging.Formatter(
        "[%(asctime)s] %(levelname)s [%(name)s.%(funcName)s:%(lineno)d] %(message)s"
    )
    log_stream.setFormatter(formatter)
    logger.addHandler(log_stream)
    logger.propagate = False
    return logger



from __future__ import annotations

import json
from typing import Optional, Iterable

from psnawp_api.core.psnawp_exceptions import (
    PSNAWPNotFound,
    PSNAWPBadRequest,
    PSNAWPForbidden,
)
from psnawp_api.models.user import User
from psnawp_api.utils.endpoints import BASE_PATH, API_PATH
from psnawp_api.utils.request_builder import RequestBuilder


class Group:
    def __init__(
        self,
        request_builder: RequestBuilder,
        group_id: Optional[str],
        users: Optional[Iterable[User]],
    ):
        """Constructor of Group. Responsible for managing messages groups and sending messages.

        .. note::

            This class is intended to be interfaced with through PSNAWP.

        :param request_builder: The instance of RequestBuilder. Used to make
            HTTPRequests.
        :type request_builder: RequestBuilder
        :param group_id: The Group ID of a group.
        :type group_id: Optional[str]
        :param users: A list of users of the members in the group.
        :type users: Optional[Iterable[User]]

        :raises: ``PSNAWPNotFound`` If group id does not exist or is invalid.

        :raises: ``PSNAWPForbidden`` If you are Dming a user who has blocked you.
            blocked you.

        """

        self._request_builder = request_builder
        self.group_id = group_id
        self.users = users

        if self.group_id is not None:
            self.get_group_information()
        elif self.users is not None:
            self._create_group()

    def _create_group(self):
        """Creates a new group if it doesn't exist. Doesn't work if user's privacy settings block invites.

        :raises: ``PSNAWPForbidden`` If you are Dming a user who has blocked you.

        """
        if self.users is not None:
            invitees = [{"accountId": user.account_id} for user in self.users]
            data = {"invitees": invitees}

            try:
                response = self._request_builder.post(
                    url=f"{BASE_PATH['gaming_lounge']}{API_PATH['create_group']}",
                    data=json.dumps(data),
                ).json()
                self.group_id = response["groupId"]
            except PSNAWPForbidden as forbidden:
                raise PSNAWPForbidden(
                    "The group cannot be created because you have been blocked by other user."
                ) from forbidden

    def change_name(self, group_name: str) -> None:
        """Changes the group name to one specified in arguments.

        .. note::

            You cannot change the name of DM groups. i.e. Groups with only two people
            (including you).

        :param group_name: The name of the group that will be set.
        :type group_name: str

        :returns: None

        :raises: ``PSNAWPBadRequest`` If you are not part of group or the group is a DM.

        """

        data = {"groupName": {"value": group_name}}
        try:
            self._request_builder.patch(
                url=f"{BASE_PATH['gaming_lounge']}{API_PATH['group_settings'].format(group_id=self.group_id)}",
                data=json.dumps(data),
            )
        except PSNAWPBadRequest as bad_req:
            raise PSNAWPBadRequest(
                f"The group name of Group ID {self.group_id} does cannot be changed. Group is either a dm or does not exist."
            ) from bad_req

    def get_group_information(self):
        """Gets the group chat information such as about me, avatars, languages etc...

        :returns: A dict containing info similar to what is shown below:
        :rtype: dict[str, Any]

        :raises: ``PSNAWPNotFound`` If group id does not exist or is invalid.

        .. code-block:: json

            {
                "groupId": "~25C4C5406FD6D50E.763F9A1EB6AB5790",
                "groupType": 0,
                "modifiedTimestamp": "1663911908531",
                "groupName": {
                    "value": "",
                    "status": 0
                },
                "groupIcon": {
                    "status": 0
                },
                "joinedTimestamp": "1616356026000",
                "isFavorite": false,
                "existsNewArrival": false,
                "mainThread": {
                    "threadId": "~25C4C5406FD6D50E.763F9A1EB6AB5790",
                    "modifiedTimestamp": "1663911908531",
                    "latestMessage": {
                        "messageUid": "1#425961448584099",
                        "messageType": 1,
                        "alternativeMessageType": 1,
                        "body": "Hello World",
                        "createdTimestamp": "1663911908531",
                        "sender": {
                            "accountId": "8520698476712646544",
                            "onlineId": "VaultTec_Trading"
                        }
                    },
                    "readMessageUid": "1#425961448584099"
                },
                "members": [
                    {
                        "accountId": "2721516955383551246",
                        "onlineId": "VaultTec-Co"
                    },
                    {
                        "accountId": "8520698476712646544",
                        "onlineId": "VaultTec_Trading"
                    }
                ],
                "partySessions": [],
                "notificationSetting": {
                    "isMute": false
                }
            }

        """

        param = {
            "includeFields": "groupName,groupIcon,members,mainThread,joinedTimestamp,modifiedTimestamp,isFavorite,existsNewArrival,partySessions,"
            "notificationSetting"
        }

        try:
            response = self._request_builder.get(
                url=f"{BASE_PATH['gaming_lounge']}{API_PATH['group_members'].format(group_id=self.group_id)}",
                params=param,
            ).json()

            return response
        except PSNAWPNotFound as not_found:
            raise PSNAWPNotFound(
                f"Group ID {self.group_id} does not exist."
            ) from not_found

    def send_message(self, message: str) -> dict[str, str]:
        """Sends a message in the group.

        .. note::

            For now only text based messaging is supported.

        :param message: Message Body

        :returns: A dict containing info similar to what is shown below:
        :rtype: dict[str, str]

        .. code-block:: json

            {
                "messageUid": "1#425961448584099",
                "createdTimestamp": "1663911908531"
            }

        """

        data = {"messageType": 1, "body": message}

        response: dict[str, str] = self._request_builder.post(
            url=f"{BASE_PATH['gaming_lounge']}{API_PATH['send_group_message'].format(group_id=self.group_id)}",
            data=json.dumps(data),
        ).json()

        return response

    def get_conversation(self, limit: int = 20):
        """Gets the conversations in a group.

        :param limit: The number of conversations to receive.
        :type limit: int

        :returns: A dict containing info similar to what is shown below:
        :rtype: dict[str, Any]

            .. code-block::

                {
                    "messages": [
                        {
                            "messageUid": "1#425961448584099",
                            "messageType": 1,
                            "alternativeMessageType": 1,
                            "body": "Hello World",
                            "createdTimestamp": "1663911908531",
                            "sender": {
                                "accountId": "8520698476712646544",
                                "onlineId": "VaultTec_Trading"
                            }
                        }
                    ],
                    "previous": "1#425961448584099",
                    "next": "1#425961448584099",
                    "reachedEndOfPage": false,
                    "messageCount": 1
                }

        """

        param = {"limit": limit}

        response = self._request_builder.get(
            url=f"{BASE_PATH['gaming_lounge']}{API_PATH['conversation'].format(group_id=self.group_id)}",
            params=param,
        ).json()

        return response

    def leave_group(self) -> None:
        """Leave the current group

        :raises: ``PSNAWPNotFound`` If you are not part of the group.

        """

        self._request_builder.delete(
            url=f"{BASE_PATH['gaming_lounge']}{API_PATH['leave_group'].format(group_id=self.group_id)}"
        )



from __future__ import annotations

import json

from psnawp_api.utils.endpoints import BASE_PATH, API_PATH
from psnawp_api.utils.request_builder import RequestBuilder


class Search:
    def __init__(self, request_builder: RequestBuilder):
        """The Search class provides the information and methods for searching resources on playstation network.

        .. note::

            This class is intended to be interfaced with through PSNAWP.

        :param request_builder: The instance of RequestBuilder. Used to make
            HTTPRequests.
        :type request_builder: RequestBuilder

        """
        self._request_builder = request_builder

    def universal_search(self, search_query: str, limit: int = 20):
        """Searches the Playstation Website. Note: It does not work as of now and the endpoints returns whole html page.

        .. note::

            Pagination not yet supported. The max number of results returned will be 20.

        :param search_query: search query
        :type search_query: str
        :param limit: Limit of number of results
        :type limit: int

        :returns: A dict containing info similar to what is shown below (Not all values
            are shown because of space limitations):
        :rtype: dict[str, Any]

            .. code-block:: json

                {
                    "prefix": "GTA",
                    "suggestions": [],
                    "domainResponses": [
                        {
                            "domain": "ConceptGameMobileApp",
                            "domainTitle": "Results for ConceptGameMobileApp with search term GTA",
                            "domainTitleMessageId": "msgid_null",
                            "domainTitleHighlight": [
                                "Results for ConceptGameMobileApp with search term GTA"
                            ],
                            "zeroState": false,
                            "univexId": "search.no_experiment.non.0.non_",
                            "facetOptions": [],
                            "next": "",
                            "totalResultCount": 33,
                            "results": [
                                {
                                    "id": "201930:UP1004-CUSA00419_00-GTAVDIGITALDOWNL",
                                    "type": "conceptProduct",
                                    "univexId": "search.no_experiment.non.0.non_",
                                    "score": 234.40508,
                                    "conceptProductMetadata": {}
                                }
                            ]
                        }
                    ],
                    "fallbackQueried": false
                }

        """

        data = {
            "searchTerm": search_query,
            "domainRequests": [
                {
                    "domain": "ConceptGameMobileApp",
                    "pagination": {"cursor": "", "pageSize": limit},
                    "featureFlags": {"isSpartacusEnabled": True},
                }
            ],
            "countryCode": "us",
            "languageCode": "en",
            "age": 99,
        }
        response = self._request_builder.post(
            url=f"{BASE_PATH['universal_search']}", data=json.dumps(data)
        ).json()
        return response

    def get_title_details(self, title_id: str):
        """get a title details using its code (eg. CUSA05486_00) as source.

        :param title_id: unique id of game.
        :type title_id: str

        :returns: A dict containing info similar to what is shown below (Not all values
            are shown because of space limitations):
        :rtype: dict[str, Any]

            .. code-block:: json

                [
                    {
                        "id": 201930,
                        "name": "Grand Theft Auto V (PlayStation®5)",
                        "nameEn": "Grand Theft Auto V (PlayStation®5)",
                        "publisherName": "Rockstar Games",
                        "media": {
                        },
                        "descriptions": [],
                        "categorizedProducts": [],
                        "titleIds": [],
                        "country": "US",
                        "language": "en",
                        "type": "GAME",
                        "localizedName": {},
                        "localizedDescriptions": {},
                        "localizedPublisherName": {},
                        "defaultProduct": {},
                        "genres": [],
                        "localizedGenres": [],
                        "subGenres": [],
                        "localizedSubGenres": [],
                        "minimumAge": 17,
                        "releaseDate": {},
                        "isRecommendableOnStore": true,
                        "isSearchableOnStore": true,
                        "storeFronts": [],
                        "supportedStoreFronts": [],
                        "marketingStories": [],
                        "compatibilityNotices": [],
                        "localizedMedia": {},
                        "revisionIds": {},
                        "contentRating": {}
                    }
                ]

        """

        param = {"age": 99, "country": "US", "language": "en-US"}

        response = self._request_builder.get(
            url=f"{BASE_PATH['game_titles']}{API_PATH['title_concept'].format(title_id=title_id)}",
            params=param,
        ).json()

        return response
